@@ -12,6 +12,7 @@
 struct slurp_box {
 	int32_t x, y;
 	int32_t width, height;
+	struct wl_list link;
 };
 
 struct slurp_state {
@@ -34,6 +35,8 @@ struct slurp_state {
 
 	uint32_t border_weight;
 	bool display_dimensions;
+	bool single_point;
+	struct wl_list boxes; // slurp_box::link
 
 	struct slurp_box result;
 };
@@ -78,8 +81,9 @@ struct slurp_seat {
 	struct slurp_output *current_output;
 	int32_t x, y;
 	int32_t pressed_x, pressed_y;
+	struct slurp_box selection;
+	bool has_selection;
 };
 
-void seat_get_box(struct slurp_seat *seat, struct slurp_box *result);
 bool box_intersect(const struct slurp_box *a, const struct slurp_box *b);
 #endif
